#!/usr/bin/env pwsh

<#
.SYNOPSIS
Calls Azure Function to do Attestation in Microsoft Subscription.

.DESCRIPTION
After compilation it calls a well-known end point to invoke the attestation service that creates my bitstream.


.PARAMETER StorageAccountName
Name of the storage account holding the client/customer's netlist file.
Alias -sa

.PARAMETER Container
Name of the blob container in the specified storage account that contains the client/customer's netlist file.
Alias -cn

.PARAMETER NetlistName
Name of the netlist file in the specified blob container.
Alias -nl

.PARAMETER BlobContainerSAS
SAS token generated by the client/customer with Read/Write access to the specified container.
Alias -cs

.EXAMPLE 
Calls the function to do attestation:

C:\> Run-ValidationFunction -StorageAccountName attestfpga -Container helloworld -NetlistName helloworld.xclbin -BlobContainerSAS $sas
where 
$sas=$(az storage container generate-sas --account-name attestfpga --name helloworld --https-only --permissions rw --expiry 2020-07-24T17:46Z -o tsv)

#>

Param(
    [Parameter(Mandatory = $true)] [Alias('sa')] [string] $StorageAccountName,
    [Parameter(Mandatory = $true)] [Alias('cn')] [string] $Container,
    [Parameter(Mandatory = $true)] [Alias('nl')] [string] $NetlistName,
    [Parameter(Mandatory = $true)] [Alias('cs')] [string] $BlobContainerSAS
)

$FunctionUrl = "https://fpga-attestation.azurewebsites.net/api/ComputeFPGA_HttpStart"

$AccountInforamtion = $(az account show)
if ($null -eq $AccountInforamtion) {
    Write-Error -Message "Failed to get account information" -Category InvalidArgument
    Write-Warning 'Check if you are already logged in to the appropriate Azure account'
    exit 1
}

    $TenantId = $(az account show --output tsv --query tenantId)
    $SubscriptionId = $(az account show --output tsv --query id)

try {
    $RequestBodyProperties = @{
        ClientSubscriptionId = $SubscriptionId;
        ClientTenantId = $TenantId;
        StorageAccountName = $StorageAccountName;
        Container = $Container;
        NetlistName = $NetlistName;
        BlobContainerSAS = $BlobContainerSAS;
    } | ConvertTo-Json| % { [System.Text.RegularExpressions.Regex]::Unescape($_) }

    Write-Output "$(Get-Date -Format "MM/dd/yyyy:HH:mm:ss") Submitting request to service..."
    $postResult = Invoke-WebRequest -Uri $FunctionUrl -Method POST  -ContentType "application/json" -Body $RequestBodyProperties -UseBasicParsing
}
catch {
    $body = $error[0].ErrorDetails.Message
    $StatusCode = [int] $error[0].Exception.Response.StatusCode
    $ReasonPhrase = $error[0].Exception.Response.ReasonPhrase
    Write-Error "Error starting the attestation function"
    Write-Error "${StatusCode} - ${ReasonPhrase}"
    Write-Error $body
}

$startResult = ConvertFrom-Json $postResult.Content

if ($null -ne $startResult.errorMessage) {
    Write-Output "Could not start attestation"
    Write-Output $startResult.errorMessage
    exit 1
}

Write-Output "OrchestrationId of your attestation request: " $startResult.instanceId 
.\Monitor-Validation.ps1 $startResult.instanceId 